SellOMatic = LibStub("AceAddon-3.0"):NewAddon("SellOMatic", "AceEvent-3.0", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SellOMatic")

-- Addon variables
local bag, merchant, slot, SOMNone
local junk_list
local button_offset_x = 20
local button_offset_y = 5
local button_offset_size = 20

function SellOMatic:OnInitialize()
	SellOMatic:Create_SOMNone()
end

-- ENABLE / DISABLE FUNCTIONS

function SellOMatic:OnEnable()
	SellOMatic:RegisterEvent("MERCHANT_SHOW")
	SellOMatic:RegisterEvent("MERCHANT_CLOSED")
	SellButton:Hide()
end

function SellOMatic:OnDisable()
	SellOMatic:UnregisterEvent("MERCHANT_SHOW")
	SellOMatic:UnregisterEvent("MERCHANT_CLOSED")
end

-- MERCHANT FUNCTIONS

function SellOMatic:MERCHANT_SHOW()
	merchant = true
	SellOMatic:CheckButtons()
	SellButton:Show()
	if SellOMatic:HasJunk() == false then SOMNone:Show() end
end

function SellOMatic:MERCHANT_CLOSED()
	merchant = false
	SellButton:Hide()
	SOMNone:Hide()
end

-- MAIN ADDON FUNCTIONS

function SellOMatic:CheckMerchant()
	local merchant_items = GetMerchantNumItems()
	if merchant and merchant_items > 0 then return true end
	return false
end

function SellOMatic:GetProfit(scan)
	local profit = 0
	if scan then SellOMatic:ScanJunk() end
	for i = 1, #junk_list do
		profit = profit + junk_list[i][4]
	end
	return profit
end

function SellOMatic:HasJunk()
	SellOMatic:ScanJunk()
	if #junk_list > 1 then
		return true
	end
	return false
end

function SellOMatic:ScanJunk()
	junk_list = {}
	for bag = 0, NUM_BAG_SLOTS, 1 do
		for slot = 1, GetContainerNumSlots(bag), 1 do
			local texture, itemCount, _, quality, _, _, itemLink, _ = GetContainerItemInfo(bag, slot)
			if texture ~= nil and quality == 0 then
				local _, _, _, _, _, _, _, _, _, _, itemSellPrice = GetItemInfo(itemLink)
				itemSellPrice = itemSellPrice * itemCount
				junk_list[#junk_list + 1] = {bag, slot, itemLink, itemSellPrice}
			end
		end
	end
end

function SellOMatic:Sell()
	if SellOMatic:HasJunk() then
		SellOMatic:SellJunk()
		SOMNone:Show()
	end
end

function SellOMatic:SellJunk()
	local num = 0
	if SellOMatic:CheckMerchant() then
		for i = 1, #junk_list do
			if not CursorHasItem() then
				num = num + 1
				UseContainerItem(junk_list[i][1], junk_list[i][2])
				SellOMatic:Print(L["Selling"]..": "..junk_list[i][3])
			end
		end
	end
	SellOMatic:Print(num.." "..L["item(s) sold"])
	SellOMatic:Print(L["You've earned"]..": "..GetCoinText(SellOMatic:GetProfit(false)," "))
end

-- MERCHANT BUTTON FUNCTIONS

function SellOMatic:Create_SOMNone()
	SOMNone = CreateFrame("Frame","SOMNone",SellButton)
	SOMNone:ClearAllPoints()
	SOMNone:SetPoint("TOPRIGHT",SellButton,"TOPRIGHT",0,0)
	SOMNone:SetHeight(32)
	SOMNone:SetWidth(32)
	SOMNone.texture = SOMNone:CreateTexture()
	SOMNone.texture:SetAllPoints(SOMNone)
	SOMNone.texture:SetColorTexture(0.3,0.3,0.3,0.7)
	SOMNone:Hide()
end

function SellOMatic:SOMButtonTooltip_Show()
	GameTooltip_SetDefaultAnchor(GameTooltip,SellButton)
	GameTooltip:ClearAllPoints()
	GameTooltip:SetPoint("TOPLEFT",SellButton,"TOPRIGHT",-30,40)
	GameTooltip:SetText(GetCoinTextureString(SellOMatic:GetProfit(true)),1,1,1)
	GameTooltip:Show()
end

function SellOMatic:SOMButtonTooltip_Hide()
	GameTooltip:Hide()
end

function SellOMatic:CheckGnomish()
	for i = 1, GetNumAddOns() do
		local name, _, _, loadable, _, _, _ = GetAddOnInfo(i)
		if name == "GnomishVendorShrinker" and loadable then
			return true
		end
	end
	return false
end

function SellOMatic:CheckButtons()
	local _, _, _, x, y = SellButton:GetPoint()
	if SellOMatic:CheckGnomish() then
		if x ~= button_offset_x or y ~= button_offset_y then
			-- Move button to base
			SellOMatic:MoveButton("offset")
		end
	else
		if x == button_offset_x or y == button_offset_y then
			-- Move button to offset
			SellOMatic:MoveButton("base")
		end
	end
end


function SellOMatic:MoveButton(where)
	if where == "base" then
		SellButton:ClearAllPoints()
		SellButton:SetPoint("TOPLEFT","MerchantFrame","TOPLEFT",60,-25)
		SellButton:SetWidth(32)
		SellButton:SetHeight(32)
		SOMNone:SetHeight(32)
		SOMNone:SetWidth(32)
	elseif where == "offset" then
		SellButton:ClearAllPoints()
		SellButton:SetPoint("BOTTOMLEFT","MerchantFrame","BOTTOMLEFT",button_offset_x,button_offset_y)
		SellButton:SetWidth(button_offset_size)
		SellButton:SetHeight(button_offset_size)
		SOMNone:SetHeight(button_offset_size)
		SOMNone:SetWidth(button_offset_size)
	end
end
